#ifndef EZ_CUDA_H_
#define EZ_CUDA_H_

#include <cuda.h>

/**
 * This macro checks return value of the CUDA runtime call and exits
 * the application if the call failed.
 */
#define ezcuda_check(value)                                              \
	{                                                                    \
		cudaError_t err = value;                                         \
		if (err != cudaSuccess)                                          \
		{                                                                \
			cerr << endl;                                                \
			cerr << "============================================\n";    \
			cerr << "Error: " << cudaGetErrorString(err) << " at line "; \
			cerr << __LINE__ << " in file " << __FILE__;                 \
			cerr << endl;                                                \
			exit(EXIT_FAILURE);                                          \
		}                                                                \
	}

#define ezcuda_check_kernel()                                                   \
	{                                                                           \
		cudaError_t err = cudaGetLastError();                                   \
		if (err != cudaSuccess)                                                 \
		{                                                                       \
			cerr << endl;                                                       \
			cerr << "============================================\n";           \
			cerr << "Kernel Error: " << cudaGetErrorString(err) << " at line "; \
			cerr << __LINE__ << " in file " << __FILE__;                        \
			cerr << endl;                                                       \
			exit(EXIT_FAILURE);                                                 \
		}                                                                       \
	}

#define D2H cudaMemcpyDeviceToHost
#define H2D cudaMemcpyHostToDevice

#define ezcudaMalloc(ptr, size) ezcuda_check(cudaMalloc(ptr, size))
#define ezcudaMemcpy(dst, src, size, way) ezcuda_check(cudaMemcpy(dst, src, size, way))
#define ezcudaFree(ptr) ezcuda_check(cudaFree(ptr))

#endif
