/*
 * gpu_timer.h
 *
 *  Created on: Sep 15, 2017
 *      Author: Jean-Michel Richer
 */

#ifndef GPU_TIMER_H_
#define GPU_TIMER_H_

#include <cuda.h>
#include <iostream>
using namespace std;
#include "ez_cuda.h"

class GPUTimer
{
private:
	cudaEvent_t t_start, t_stop;

public:
	GPUTimer()
	{
		ezcuda_check(cudaEventCreate(&t_start));
		ezcuda_check(cudaEventCreate(&t_stop));
	}

	~GPUTimer()
	{
		ezcuda_check(cudaEventDestroy(t_start));
		ezcuda_check(cudaEventDestroy(t_stop));
	}

	void start()
	{
		ezcuda_check(cudaEventRecord(t_start, NULL));
	}

	void stop()
	{
		ezcuda_check(cudaEventRecord(t_stop, NULL));
		ezcuda_check(cudaEventSynchronize(t_stop));
	}

	float elapsed()
	{
		float milli_seconds = 0.0f;
		ezcuda_check(cudaEventElapsedTime(&milli_seconds, t_start, t_stop));
		return milli_seconds;
	}

	ostream &print(ostream &out)
	{
		float milli_seconds = 0.0f;
		ezcuda_check(cudaEventElapsedTime(&milli_seconds, t_start, t_stop));
		out << fixed;
		out << milli_seconds;
		return out;
	}

	friend ostream &operator<<(ostream &out, GPUTimer &g)
	{
		return g.print(out);
	}
};

#endif /* GPU_TIMER_H_ */
