#include <iostream>
#include <vector>
#include <algorithm>
#include <stdexcept>
using namespace std;
#include <getopt.h>
#include "cpu_timer.h"
#include "cpu_bitmap.h"
#include "complex.h"

#include "ez_cuda.h"
#include "gl_animation.h"


// ==================================================================
// global variables
// ==================================================================
int image_width = 1024;
int image_height = 1024;
bool view_flag = false;
int iterations = 200;
REAL constant_real = 0.3;
REAL constant_imag = 0.5;

const REAL xmin = -2.4;
const REAL xmax = 2.4;
const REAL ymin = -1.5;
const REAL ymax = 1.5;
const REAL NORM_THRESOLD = 4.0;

GLImage *image;

// ==================================================================
// program command line arguments
// ==================================================================
/*
static struct option long_options[] = {
		{"width"		, required_argument, 0, 'w'},
		{"height"		, required_argument, 0, 'h'},
		{"view"			, 		no_argument, 0, 'v'},
		{"iterations"	, required_argument, 0, 'i'},
		{"constant-real", required_argument, 0, 'r'},
		{"constant-imag", required_argument, 0, 's'},
		{0,0,0,0}
};
*/

/**
 * determine if given point is in Julia Set
 *
 */
REAL julia(int x, int y, int max_iter) {
	REAL r = x * (REAL)(xmax - xmin)/image_width + xmin;
	REAL i = y * (REAL)(ymax - ymin)/image_height + ymin;

	aComplex z(r, i);
	aComplex c(constant_real, constant_imag);

	for (int k=0; k<max_iter; k++) {
		z = z * z + c;
		
		/*if (z.norm() > NORM_THRESOLD)  {
			return NORM_THRESOLD;
		}*/
	}

	return z.norm();
}


void kernel(ImagePoint *img, int max_iter){

	//#pragma omp parallel for
	for (int y=0; y<image_height; y++) {
		for (int x=0; x<image_width; x++) {

			int offset = x + y * image_width;
			REAL juliaValue = julia(x, y, max_iter );

			int  color = std::round(juliaValue * 255);
			color = 255 - color;
			img[offset].red = color;
			img[offset].green = color;
			img[offset].blue = color;
			img[offset].alpha = 255;
		}
	}
}

/**
 * subprogram called for the animation
 */
void animate() {
	// exchange input/output cells
}

/**
 * clean up memory allocated on the GPU
 */
void leave() {
}

int main(int argc, char *argv[]) {
	
	while (true) {
		
		int c = getopt(argc, argv, "w:h:i:r:s:v");
		if (c == -1) break;

		switch(c) {

		case 'w':
			image_width = atoi(optarg);
			if ((image_width <= 128) || (image_width >= 8192)) {
				throw std::invalid_argument("bad image width : 128..8192");
			}
			break;

		case 'h':
			image_height = atoi(optarg);
			if ((image_height <= 128) || (image_height >= 8192)) {
				throw std::invalid_argument("bad image height : 128..8192");
			}
			break;

		case 'v':
			view_flag = true;
			break;

		case 'i':
			iterations = atoi(optarg);
			if ((iterations <= 100) || (iterations >= 10000000)) {
				throw std::invalid_argument("bad number of iterations : 100..10000000");
			}
			break;

		case 'r':
			constant_real = atof(optarg);
			break;

		case 's':
			constant_imag = atof(optarg);
			break;
		}
	}

	cout << "image_width=" << image_width << endl;
	cout << "image_height=" << image_height << endl;
	cout << "iterations=" << iterations << endl;
	cout << "constant_real=" << constant_real << endl;
	cout << "constant_imag=" << constant_imag << endl;

	/*
	image = new GLImage(image_height, image_width);
	GLAnimation::instance().image(image);


	CPUTimer c_timer; 
	c_timer.start();
	kernel(image->pixels(), iterations);
	c_timer.stop();
	cout << "cpu.time=" << c_timer << endl;

	GLAnimation::instance().run(argc, argv, animate,  leave);
	*/

	CPUBitmap bitmap(image_width, image_height);
	
	CPUTimer c_timer;
	c_timer.start();
	
	kernel((ImagePoint *) bitmap.get_ptr(), iterations);
	c_timer.stop();
	
	cout << "cpu time(ms)=" << c_timer << endl;
	

	if (view_flag) {
		bitmap.display_and_exit();
	}

	return EXIT_SUCCESS;
}

